__version__ = "2.0.0"

from .AnimeSaturn import AnimeSaturn
from .StreamingCommunity import StreamingCommunity
from .JWPlayer import JWPlayer

__all__ = ['AnimeSaturn', 'StreamingCommunity', 'JWPlayer']
